# -*- coding: UTF-8 -*-
from __future__ import unicode_literals, division

import attr

from colour import Color

from .utils import group_nested, image_colors, get_rgb_image, resize_image


@attr.s(slots=True)
class ImageToKatex(object):
    background = attr.ib(type=Color)
    max_length = attr.ib(type=int)
    fontsize = attr.ib(type=float)
    precision = attr.ib(5, type=float)
    _pixelsize = attr.ib(None, type=float)
    _katex_multiplier = attr.ib(1.21, type=float)
    _unit_multiplier = attr.ib(0.1, type=float)

    formatters = dict(
        cell="\\color{{{}}}\\rule{}pt{}pt".format,
        columns="".join,
        rows_spacer="\\\\[-{}pt]".format,
        table="\\(\\begin{{aligned}}{}\\end{{aligned}}\\)".format,
    )

    @property
    def pixelsize(self):
        if self._pixelsize is None:
            self._pixelsize = 1 / (
                self.fontsize * self._katex_multiplier * self._unit_multiplier
            )
        return self._pixelsize

    @pixelsize.setter
    def pixelsize(self, value):
        self._pixelsize = value

    def in_background(self, color):
        # TODO: Make this work for colors close to the background color
        return color == self.background

    def get_float_value(self, number):
        return str(float(round(number, self.precision))).strip("0").rstrip(".")

    def _row(self, row, height):
        formatted_height = self.get_float_value(height)
        for pixel, equal_pixels in row:
            width = self.pixelsize * equal_pixels
            if self.in_background(pixel):
                color = "transparent"
            else:
                color = pixel.hex
            while width > 0:
                formatted_width = self.get_float_value(min(width, self.max_size))
                yield self.formatters["cell"](color, formatted_width, formatted_height)
                width -= self.max_size

    def _rows(self, rows):
        for row, equal_rows in rows:
            height = self.pixelsize * equal_rows
            while height > 0:
                row_data = self._row(row, min(height, self.max_size))
                yield self.formatters["columns"](row_data)
                height -= self.max_size

    def get(self, image):
        spacing = self.get_float_value(15 - min(self.pixelsize, 8.4))
        spacer = self.formatters["rows_spacer"](spacing)

        rows = self._rows(group_nested(image_colors(image)))

        return self.formatters["table"](spacer.join(rows))

    def get_katex(self, fp, size, resample=None):
        image = get_rgb_image(fp, self.background)
        im = resize_image(image, size, resample=resample)
        return self.get(im)

    def get_largest_latex(self, fp, resample=None):
        old, new = "", ""
        size = 0
        image = get_rgb_image(fp, self.background)
        while len(new) < self.max_length:
            old = new
            size += 1
            im = resize_image(image, size, resample=resample)
            new = self.get(new_im)
        return old, size



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals

from .core import *
from .models import *

# Remember to update this in setup.py as well
__version__ = "0.0.2"
__author__ = "Mads Marquart"
__author_email__ = "madsmtm@gmail.com"
__copyright__ = "Copyright 2018 by Mads Marquart"
__license__ = "BSD 3-Clause"



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals

import click

from colour import Color

from .core import ImageToKatex
from .models import ImageToKatexFacebook

CLASSES = {"facebook": ImageToKatexFacebook}


class ColorTypeBase(click.ParamType):
    name = "color"

    def convert(self, value, param, ctx):
        try:
            if not value.startswith("#"):
                value = "#{}".format(value)
            return Color(value)
        except Exception as e:
            self.fail("%s is not a valid color" % e)


ColorType = ColorTypeBase()


@click.command()
@click.argument("image", type=click.File(mode="rb"))
@click.option(
    "-s",
    "--size",
    default=16,
    type=int,
    help="The amount of pixels to use. The image will keep it's aspect ratio",
    show_default=True,
)
@click.option(
    "-t",
    "--type",
    "type_",
    default="facebook",
    type=click.Choice(tuple(CLASSES)),
    help="The engine to use. Currently this is only built for Facebook",
)
@click.option(
    "-ps",
    "--pixelsize",
    type=float,
    help="The size of a single pixel, in the unit `pt`. Can reduce output size if set",
    show_default="single pixel on the screen",
)
@click.option(
    "-f",
    "--fontsize",
    type=int,
    help="Facebook's font-size in pixels to take into account",
)
@click.option(
    "-b",
    "--background",
    type=ColorType,
    help="Background color of the image, in hexadecimal",
)
@click.option(
    "-r",
    "--resample",
    type=click.IntRange(min=0, max=3),
    help="Resampling level when resizing. Esentially means level of anti-aliasing",
)
@click.option(
    "-p",
    "--precision",
    type=int,
    help="How precise the pixel-sizes should be. Can reduce output size",
)
@click.pass_context
def cli(ctx, type_, image, size, resample=None, **kwargs):
    """Convert an image into KaTeX using \\rule"""
    kwargs = dict(filter(lambda item: item[1] is not None, kwargs.items()))
    image_to_katex = CLASSES[type_](**kwargs)

    if False:
        katex, size = image_to_katex.get_largest_latex(
            image, args.pixelsize, args.fontsize, args.precision, args.resample
        )
    else:
        katex = image_to_katex.get_katex(image, size, resample=resample)

    length = "{}/{}".format(len(katex), image_to_katex.max_length)
    if len(katex) > image_to_katex.max_length:
        ctx.fail("Length of resulting string too large: {}".format(length))

    click.echo(katex)
    click.echo("Total length of string: {}".format(length))



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals, division

from PIL import Image
from colour import Color


def color_to_int(color):
    return tuple(int(round(x * 255)) for x in color.get_rgb())


def color_from_int(ints):
    color = Color()
    color.set_rgb(tuple(x / 255 for x in ints))
    return color


def get_rgb_image(fp, background):
    raw = Image.open(fp).convert("RGBA")
    bg = Image.new(raw.mode, raw.size, color=color_to_int(background))
    return Image.alpha_composite(bg, raw).convert("RGB")


def resize_image(image, size, resample=None):
    if resample is None:
        resample = Image.NEAREST

    # Preserve aspect ratio
    ratio = min(size / image.width, size / image.height)
    height = int(round(image.height * ratio))
    width = int(round(image.width * ratio))

    return image.resize((height, width), resample=resample)


def image_colors(image):
    for y in range(image.height):
        # Not yield from, since we want the result as a generator
        yield (color_from_int(image.getpixel((x, y))) for x in range(image.width))


def group(iterable):
    """Groups items in an iterable

    Example:
        >>> list(group([1, 2, 2, 1, 1, 3]))
        [(1, 1), (2, 2), (1, 2), (3, 1)]
    """
    iterable = iter(iterable)
    amount_equal = 1
    prev = next(iterable)
    for item in iterable:
        if prev != item:
            yield prev, amount_equal
            amount_equal = 0
        amount_equal += 1
        prev = item
    yield prev, amount_equal


def group_nested(nested_iterable):
    """A nested version of `group`"""
    yield from group(map(tuple, map(group, nested_iterable)))



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals

import attr

from colour import Color
from .core import ImageToKatex


@attr.s(slots=True)
class ImageToKatexFacebook(ImageToKatex):
    background = attr.ib(Color("#f1f0f0"), type=Color)
    max_length = attr.ib(20000, type=int)
    fontsize = attr.ib(14, type=float)
    max_size = attr.ib(50, type=int)



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals

from setuptools import setup
from os import path

# Read contents of the readme file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name="katex",
    version="0.0.2", # Remember to update this in katex/__init__.py as well
    license="BSD 3-Clause",
    license_file="LICENSE",
    author="Mads Marquart",
    author_email="madsmtm@gmail.com",
    description="Convert an image into KaTeX to use in Facebook Messenger",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="Facebook KaTeX Messenger ",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Communications :: Chat",
        "Topic :: Games/Entertainment",
        "Topic :: Scientific/Engineering :: Human Machine Interfaces",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities",
    ],
    url="http://github.com/madsmtm/katex",
    packages=["katex"],
    python_requires=">=3.4, <4.0",
    install_requires=["Pillow>=5.2.0", "attrs", "Click", "colour"],
    entry_points={"console_scripts": ["image_to_katex=katex.cli:image_to_katex"]},
)
